"use strict";

exports.__esModule = true;
exports.createInstance = exports.useFetch = exports.useAsync = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _useAsync = require("./useAsync");

Object.defineProperty(exports, "useAsync", {
  enumerable: true,
  get: function get() {
    return _interopRequireDefault(_useAsync).default;
  }
});
Object.defineProperty(exports, "useFetch", {
  enumerable: true,
  get: function get() {
    return _useAsync.useFetch;
  }
});

var _react = require("react");

var _react2 = _interopRequireDefault(_react);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var isFunction = function isFunction(arg) {
  return typeof arg === "function";
};

/**
 * createInstance allows you to create instances of Async that are bound to a specific promise.
 * A unique instance also uses its own React context for better nesting capability.
 */
var createInstance = exports.createInstance = function createInstance() {
  var defaultProps = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var displayName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "Async";

  var _React$createContext = _react2.default.createContext(),
      Consumer = _React$createContext.Consumer,
      Provider = _React$createContext.Provider;

  var Async = function (_React$Component) {
    _inherits(Async, _React$Component);

    function Async(props) {
      _classCallCheck(this, Async);

      var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

      _this.start = _this.start.bind(_this);
      _this.load = _this.load.bind(_this);
      _this.run = _this.run.bind(_this);
      _this.cancel = _this.cancel.bind(_this);
      _this.onResolve = _this.onResolve.bind(_this);
      _this.onReject = _this.onReject.bind(_this);
      _this.setData = _this.setData.bind(_this);
      _this.setError = _this.setError.bind(_this);

      var promiseFn = props.promiseFn || defaultProps.promiseFn;
      var initialValue = props.initialValue || defaultProps.initialValue;
      var initialError = initialValue instanceof Error ? initialValue : undefined;
      var initialData = initialError ? undefined : initialValue;

      _this.mounted = false;
      _this.counter = 0;
      _this.args = [];
      _this.abortController = { abort: function abort() {} };
      _this.state = {
        initialValue: initialValue,
        data: initialData,
        error: initialError,
        isLoading: !initialValue && isFunction(promiseFn),
        startedAt: undefined,
        finishedAt: initialValue ? new Date() : undefined,
        cancel: _this.cancel,
        run: _this.run,
        reload: function reload() {
          _this.load();
          _this.run.apply(_this, _this.args);
        },
        setData: _this.setData,
        setError: _this.setError
      };
      return _this;
    }

    Async.prototype.componentDidMount = function componentDidMount() {
      this.mounted = true;
      this.state.initialValue || this.load();
    };

    Async.prototype.componentDidUpdate = function componentDidUpdate(prevProps) {
      var _props = this.props,
          watch = _props.watch,
          _props$watchFn = _props.watchFn,
          watchFn = _props$watchFn === undefined ? defaultProps.watchFn : _props$watchFn,
          promiseFn = _props.promiseFn;

      if (watch !== prevProps.watch) this.load();
      if (watchFn && watchFn(_extends({}, defaultProps, this.props), _extends({}, defaultProps, prevProps))) this.load();
      if (promiseFn !== prevProps.promiseFn) {
        if (promiseFn) this.load();else this.cancel();
      }
    };

    Async.prototype.componentWillUnmount = function componentWillUnmount() {
      this.cancel();
      this.mounted = false;
    };

    Async.prototype.start = function start() {
      if ("AbortController" in window) {
        this.abortController.abort();
        this.abortController = new window.AbortController();
      }
      this.counter++;
      this.setState({ isLoading: true, startedAt: new Date(), finishedAt: undefined });
    };

    Async.prototype.load = function load() {
      var promiseFn = this.props.promiseFn || defaultProps.promiseFn;
      if (!promiseFn) return;
      this.start();
      return promiseFn(this.props, this.abortController).then(this.onResolve(this.counter), this.onReject(this.counter));
    };

    Async.prototype.run = function run() {
      var deferFn = this.props.deferFn || defaultProps.deferFn;
      if (!deferFn) return;

      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      this.args = args;
      this.start();
      return deferFn(args, _extends({}, defaultProps, this.props), this.abortController).then(this.onResolve(this.counter), this.onReject(this.counter));
    };

    Async.prototype.cancel = function cancel() {
      this.counter++;
      this.abortController.abort();
      this.setState({ isLoading: false, startedAt: undefined });
    };

    Async.prototype.onResolve = function onResolve(counter) {
      var _this2 = this;

      return function (data) {
        if (_this2.mounted && _this2.counter === counter) {
          var onResolve = _this2.props.onResolve || defaultProps.onResolve;
          _this2.setData(data, function () {
            return onResolve && onResolve(data);
          });
        }
        return data;
      };
    };

    Async.prototype.onReject = function onReject(counter) {
      var _this3 = this;

      return function (error) {
        if (_this3.mounted && _this3.counter === counter) {
          var onReject = _this3.props.onReject || defaultProps.onReject;
          _this3.setError(error, function () {
            return onReject && onReject(error);
          });
        }
        return error;
      };
    };

    Async.prototype.setData = function setData(data, callback) {
      this.setState({ data: data, error: undefined, isLoading: false, finishedAt: new Date() }, callback);
      return data;
    };

    Async.prototype.setError = function setError(error, callback) {
      this.setState({ error: error, isLoading: false, finishedAt: new Date() }, callback);
      return error;
    };

    Async.prototype.render = function render() {
      var children = this.props.children;

      if (isFunction(children)) {
        return _react2.default.createElement(
          Provider,
          { value: this.state },
          children(this.state)
        );
      }
      if (children !== undefined && children !== null) {
        return _react2.default.createElement(
          Provider,
          { value: this.state },
          children
        );
      }
      return null;
    };

    return Async;
  }(_react2.default.Component);

  /**
   * Renders only when deferred promise is pending (not yet run).
   *
   * @prop {boolean} persist Show until we have data, even while loading or when an error occurred
   * @prop {Function|Node} children Function (passing state) or React node
   */


  var Pending = function Pending(_ref) {
    var children = _ref.children,
        persist = _ref.persist;
    return _react2.default.createElement(
      Consumer,
      null,
      function (state) {
        if (state.data !== undefined) return null;
        if (!persist && state.isLoading) return null;
        if (!persist && state.error !== undefined) return null;
        return isFunction(children) ? children(state) : children || null;
      }
    );
  };

  /**
   * Renders only while loading.
   *
   * @prop {boolean} initial Show only on initial load (data is undefined)
   * @prop {Function|Node} children Function (passing state) or React node
   */
  var Loading = function Loading(_ref2) {
    var children = _ref2.children,
        initial = _ref2.initial;
    return _react2.default.createElement(
      Consumer,
      null,
      function (state) {
        if (!state.isLoading) return null;
        if (initial && state.data !== undefined) return null;
        return isFunction(children) ? children(state) : children || null;
      }
    );
  };

  /**
   * Renders only when promise is resolved.
   *
   * @prop {boolean} persist Show old data while loading
   * @prop {Function|Node} children Function (passing data and state) or React node
   */
  var Resolved = function Resolved(_ref3) {
    var children = _ref3.children,
        persist = _ref3.persist;
    return _react2.default.createElement(
      Consumer,
      null,
      function (state) {
        if (state.data === undefined) return null;
        if (!persist && state.isLoading) return null;
        if (!persist && state.error !== undefined) return null;
        return isFunction(children) ? children(state.data, state) : children || null;
      }
    );
  };

  /**
   * Renders only when promise is rejected.
   *
   * @prop {boolean} persist Show old error while loading
   * @prop {Function|Node} children Function (passing error and state) or React node
   */
  var Rejected = function Rejected(_ref4) {
    var children = _ref4.children,
        persist = _ref4.persist;
    return _react2.default.createElement(
      Consumer,
      null,
      function (state) {
        if (state.error === undefined) return null;
        if (state.isLoading && !persist) return null;
        return isFunction(children) ? children(state.error, state) : children || null;
      }
    );
  };

  Async.Pending = Pending;
  Async.Loading = Loading;
  Async.Resolved = Resolved;
  Async.Rejected = Rejected;

  Async.displayName = displayName;
  Async.Pending.displayName = displayName + ".Pending";
  Async.Loading.displayName = displayName + ".Loading";
  Async.Resolved.displayName = displayName + ".Resolved";
  Async.Rejected.displayName = displayName + ".Rejected";

  return Async;
};

exports.default = createInstance();